@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,32 +9,33 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#include "extensioncontext.h"
-#include "outputcallback.h"
 #include "eventcallback.h"
+#include "extensioncontext.h"
+#include "gdbmihelpers.h"
+#include "outputcallback.h"
+#include "stringutils.h"
 #include "symbolgroup.h"
 #include "symbolgroupvalue.h"
-#include "stringutils.h"
-#include "gdbmihelpers.h"
+
+#ifdef WITH_PYTHON
+#include <Python.h>
+#include "pystdoutredirect.h"
+#include "pycdbextmodule.h"
+#endif
 
 #include <cstdio>
 #include <sstream>
@@ -111,7 +112,8 @@ enum Command {
     CmdWidgetAt,
     CmdBreakPoints,
     CmdTest,
-    CmdSetParameter
+    CmdSetParameter,
+    CmdScript
 };
 
 static const CommandDescription commandDescriptions[] = {
@@ -124,13 +126,14 @@ static const CommandDescription commandDescriptions[] = {
  "-c complex dumpers"},
 {"locals",
  "Prints local variables of symbol group in GDBMI or debug format",
- "[-t token] [-v] [T formats] [-I formats] [-f debugfilter] [-c] [-h] [-d]\n[-e expand-list] [-u uninitialized-list]\n"
+ "[-t token] [-v] [T formats] [-I formats] [-f debugfilter] [-a] [-c] [-h] [-d]\n[-e expand-list] [-u uninitialized-list]\n"
  "[-W] [-w watch-iname watch-expression] <frame-number> [iname]\n"
  "-h human-readable output\n"
  "-v increase verboseness of dumping\n"
  "-d debug output\n"
  "-f debug_filter\n"
  "-c complex dumpers\n"
+ "-a sort alphabetically\n"
  "-e expand-list        Comma-separated list of inames to be expanded beforehand\n"
  "-u uninitialized-list Comma-separated list of uninitialized inames\n"
  "-I formatmap          map of 'hex-encoded-iname=typecode'\n"
@@ -157,7 +160,7 @@ static const CommandDescription commandDescriptions[] = {
 {"assign","Assigns a value to a variable in current symbol group.",
  "[-t token] [-h] <iname=value>\n"
  "-h    Data are hex-encoded, binary data\n"
- "-u    Data are hex-encoded, UTF16 data"
+ "-e    iname is an hex-encoded expression to be evaluated "
 },
 {"threads","Lists threads in GDBMI format.","[-t token]"},
 {"registers","Lists registers in GDBMI format","[-t token]"},
@@ -176,7 +179,8 @@ static const CommandDescription commandDescriptions[] = {
 {"breakpoints","List breakpoints with modules","[-h] [-v]"},
 {"test","Testing command","-T type | -w watch-expression"},
 {"setparameter","Set parameter",
- "maxStringLength=value maxArraySize=value maxStackDepth=value stateNotification=1,0"}
+ "maxStringLength=value maxArraySize=value maxStackDepth=value stateNotification=1,0"},
+{"script", "Run Python command", "[-t token]"}
 };
 
 typedef std::vector<std::string> StringVector;
@@ -187,6 +191,12 @@ static inline bool isOption(const std::string &opt)
     return opt.size() == 2 && opt.at(0) == '-' && opt != "--";
 }
 
+struct CommandToken
+{
+    int majorPart = 0;
+    int minorPart = 0;
+};
+
 // Helper for commandTokens() below:
 // Simple splitting of command lines allowing for '"'-quoted tokens
 // 'typecast local.i "class QString *"' -> ('typecast','local.i','class QString *')
@@ -247,6 +257,38 @@ static inline void splitCommand(PCSTR args, Inserter it)
     }
 }
 
+// check whether arguments start with a "-t 23.1" or "-t 3" token identifier
+CommandToken extractToken(PCSTR args, PCSTR *afterToken)
+{
+    CommandToken token;
+    while (*args == ' ') // skip whitespace
+        ++args;
+    if (*args != '-')
+        return {};
+    if (*(++args) != 't')
+        return {};
+    ++args;
+    while (*args == ' ') // skip whitespace
+        ++args;
+    PSTR end = nullptr;
+    token.majorPart = strtol(args, &end, 10);
+    if (args >= end)
+        return {};
+    args = end;
+    if (*args == '.') {
+        ++args;
+        end = nullptr;
+        token.minorPart = strtol(args, &end, 10);
+        if (args >= end)
+            return {};
+        args = end;
+    }
+    while (*args == ' ') // skip whitespace
+        ++args;
+    *afterToken = args;
+    return token;
+}
+
 // Split & Parse the arguments of a command and extract the
 // optional first integer token argument ('command -t <number> remaining arguments')
 // Each command takes the 'token' argument and includes it in its output
@@ -255,18 +297,25 @@ static inline void splitCommand(PCSTR args, Inserter it)
 template<class StringContainer>
 static inline StringContainer commandTokens(PCSTR args, int *token = 0)
 {
-    typedef StringContainer::iterator ContainerIterator;
+    static std::map<int, std::string> s_commandBuffer;
 
     if (token)
         *token = -1; // Handled as 'display' in engine, so that user can type commands
+
     StringContainer tokens;
-    splitCommand(args, std::back_inserter(tokens));
-    // Check for token
-    ContainerIterator it = tokens.begin();
-    if (it != tokens.end() && *it == "-t" && ++it != tokens.end()) {
-        if (token)
-            std::istringstream(*it) >> *token;
-        tokens.erase(tokens.begin(), ++it);
+    CommandToken commandToken = extractToken(args, &args);
+    if (commandToken.majorPart != 0) {
+        s_commandBuffer[commandToken.majorPart].append(args);
+        if (commandToken.minorPart == 0) {
+            DebugPrint() << commandToken.majorPart << ':' << s_commandBuffer[commandToken.majorPart];
+            splitCommand(s_commandBuffer[commandToken.majorPart].data(), std::back_inserter(tokens));
+            if (token)
+                *token = commandToken.majorPart;
+        } else if (token) {
+            *token = 0;
+        }
+    } else {
+        splitCommand(args, std::back_inserter(tokens));
     }
     return tokens;
 }
@@ -279,8 +328,10 @@ extern "C" HRESULT CALLBACK pid(CIDebugClient *client, PCSTR args)
 
     int token;
     commandTokens<StringList>(args, &token);
-    dprintf("Qt Creator CDB extension version 3.3 %d bit built %s.\n",
-            sizeof(void *) * 8, __DATE__);
+    if (token == 0) // partial message
+        return S_OK;
+    dprintf("Qt Creator CDB extension version 4.3 %d bit.\n",
+            sizeof(void *) * 8);
     if (const ULONG pid = currentProcessId(client))
         ExtensionContext::instance().report('R', token, 0, "pid", "%u", pid);
     else
@@ -300,6 +351,9 @@ extern "C" HRESULT CALLBACK expandlocals(CIDebugClient *client, PCSTR args)
 
     int token;
     StringList tokens = commandTokens<StringList>(args, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     StringVector inames;
     bool runComplexDumpers = false;
     do {
@@ -372,6 +426,9 @@ DumpCommandParameters::ParseOptionResult DumpCommandParameters::parseOption(Stri
     case 'c':
         dumpParameters.dumpFlags |= DumpParameters::DumpComplexDumpers;
         break;
+    case 'a':
+        dumpParameters.dumpFlags |= DumpParameters::DumpAlphabeticallySorted;
+        break;
     case 'f':
         if (options->size() < 2)
             return Error;
@@ -410,16 +467,18 @@ DumpCommandParameters::ParseOptionResult DumpCommandParameters::parseOption(Stri
 // Display local variables of symbol group in GDBMI or debug output form.
 // Takes an optional iname which is expanded before displaying (for updateWatchData)
 
-static std::string commmandLocals(ExtensionCommandContext &commandExtCtx,PCSTR args, int *token, std::string *errorMessage)
+static std::string commandLocals(ExtensionCommandContext &commandExtCtx,PCSTR args, int *token, std::string *errorMessage)
 {
     typedef WatchesSymbolGroup::InameExpressionMap InameExpressionMap;
     typedef InameExpressionMap::value_type InameExpressionMapEntry;
 
     // Parse the command
+    StringList tokens = commandTokens<StringList>(args, token);
+    if (token == 0) // partial arguments
+        return {};
     ExtensionContext &extCtx = ExtensionContext::instance();
     DumpCommandParameters parameters;
     std::string iname;
-    StringList tokens = commandTokens<StringList>(args, token);
     StringVector expandedInames;
     StringVector uninitializedInames;
     InameExpressionMap watcherInameExpressionMap;
@@ -507,16 +566,18 @@ static std::string commmandLocals(ExtensionCommandContext &commandExtCtx,PCSTR a
     // Synchronize watches if desired.
     WatchesSymbolGroup *watchesSymbolGroup = extCtx.watchesSymbolGroup();
     if (watchSynchronization) {
-        if (watcherInameExpressionMap.empty()) { // No watches..kill group.
-            watchesSymbolGroup = 0;
-            extCtx.discardWatchesSymbolGroup();
-            if (SymbolGroupValue::verbose)
-                DebugPrint() << "Discarding watchers";
-        } else {
-            // Force group into existence
-            watchesSymbolGroup = extCtx.watchesSymbolGroup(commandExtCtx.symbols(), errorMessage);
-            if (!watchesSymbolGroup || !watchesSymbolGroup->synchronize(commandExtCtx.symbols(), watcherInameExpressionMap, errorMessage))
-                return std::string();
+        watchesSymbolGroup = 0;
+        extCtx.discardWatchesSymbolGroup();
+    }
+
+    if (watchesSymbolGroup == 0
+            && (!watcherInameExpressionMap.empty() || WatchesSymbolGroup::isWatchIname(iname))) {
+        // Force group into existence
+        watchesSymbolGroup = extCtx.watchesSymbolGroup(commandExtCtx.symbols(), errorMessage);
+        if (!watchesSymbolGroup || !watchesSymbolGroup->synchronize(commandExtCtx.symbols(),
+                                                                    watcherInameExpressionMap,
+                                                                    errorMessage)) {
+            return std::string();
         }
     }
 
@@ -557,7 +618,45 @@ static std::string commmandLocals(ExtensionCommandContext &commandExtCtx,PCSTR a
         return dumpRc;
     }
 
-    return symGroup->dump(iname, dumpContext, parameters.dumpParameters, errorMessage);
+    if (WatchesSymbolGroup::isWatchIname(iname))
+        return watchesSymbolGroup->dump(iname, dumpContext, parameters.dumpParameters, errorMessage);
+    else
+        return symGroup->dump(iname, dumpContext, parameters.dumpParameters, errorMessage);
+}
+
+extern "C" HRESULT CALLBACK script(CIDebugClient *client, PCSTR argsIn)
+{
+    ExtensionCommandContext exc(client);
+    int token;
+#ifdef WITH_PYTHON
+    const StringList args = commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial arguments
+        return {};
+    std::stringstream command;
+    for (std::string arg : args)
+        command << arg << ' ';
+
+    PyObject *ptype      = NULL;
+    PyObject *pvalue     = NULL;
+    PyObject *ptraceback = NULL;
+    PyErr_Fetch(&ptype, &pvalue, &ptraceback);
+    startCapturePyStdout();
+    const char result = (PyRun_SimpleString(command.str().c_str()) == 0) ? 'R' : 'N';
+    if (PyErr_Occurred())
+        PyErr_Print();
+    ExtensionContext::instance().reportLong(result, token, "script", collectOutput().c_str());
+    endCapturePyStdout();
+    PyErr_Restore(ptype, pvalue, ptraceback);
+#else
+    commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial arguments
+        return {};
+    ExtensionContext::instance().report('N', token, 0, "script",
+            "Python is not supported in this CDB extension.\n"
+            "You need to define PYTHON_INSTALL_DIR in your creator build environment "
+            "pointing to a Python 3.5 installation.");
+#endif
+    return S_OK;
 }
 
 extern "C" HRESULT CALLBACK locals(CIDebugClient *client, PCSTR args)
@@ -565,7 +664,9 @@ extern "C" HRESULT CALLBACK locals(CIDebugClient *client, PCSTR args)
     ExtensionCommandContext exc(client);
     std::string errorMessage;
     int token;
-    const std::string output = commmandLocals(exc, args, &token, &errorMessage);
+    const std::string output = commandLocals(exc, args, &token, &errorMessage);
+    if (token == 0) // partial message
+        return S_OK;
     SymbolGroupValue::verbose = 0;
     if (output.empty())
         ExtensionContext::instance().report('N', token, 0, "locals", errorMessage.c_str());
@@ -583,6 +684,9 @@ static std::string commmandWatches(ExtensionCommandContext &exc, PCSTR args, int
     // Parse the command
     DumpCommandParameters parameters;
     StringList tokens = commandTokens<StringList>(args, token);
+    if (token == 0) // partial message
+        return {};
+
     // Parse away options
     for (bool optionLeft = true;  optionLeft && !tokens.empty(); ) {
         switch (parameters.parseOption(&tokens)) {
@@ -620,6 +724,9 @@ extern "C" HRESULT CALLBACK watches(CIDebugClient *client, PCSTR args)
     std::string errorMessage = "e";
     int token = 0;
     const std::string output = commmandWatches(exc, args, &token, &errorMessage);
+    if (token == 0) // partial message
+        return S_OK;
+
     SymbolGroupValue::verbose = 0;
     if (output.empty())
         ExtensionContext::instance().report('N', token, 0, "locals", errorMessage.c_str());
@@ -635,6 +742,9 @@ static std::string dumplocalHelper(ExtensionCommandContext &exc,PCSTR args, int
 {
     // Parse the command
     StringList tokens = commandTokens<StringList>(args, token);
+    if (token == 0) // partial message
+        return {};
+
     // Frame and iname
     unsigned frame;
     if (tokens.empty() || integerFromString(tokens.front(), &frame)) {
@@ -658,7 +768,8 @@ static std::string dumplocalHelper(ExtensionCommandContext &exc,PCSTR args, int
         return std::string();
     }
     std::wstring value;
-    if (!dumpSimpleType(n->asSymbolGroupNode(), SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()), &value)) {
+    std::string tmp;
+    if (!dumpSimpleType(n->asSymbolGroupNode(), SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()), &value, &tmp)) {
         *errorMessage = "Cannot dump " + iname;
         return std::string();
     }
@@ -671,6 +782,9 @@ extern "C" HRESULT CALLBACK dumplocal(CIDebugClient *client, PCSTR  argsIn)
     std::string errorMessage;
     int token = 0;
     const std::string value = dumplocalHelper(exc,argsIn, &token, &errorMessage);
+    if (token == 0) // partial message
+        return S_OK;
+
     if (value.empty())
         ExtensionContext::instance().report('N', token, 0, "dumplocal", errorMessage.c_str());
     else
@@ -690,6 +804,9 @@ extern "C" HRESULT CALLBACK typecast(CIDebugClient *client, PCSTR args)
 
     int token;
     const StringVector tokens = commandTokens<StringVector>(args, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     std::string iname;
     std::string desiredType;
     if (tokens.size() == 3u && integerFromString(tokens.front(), &frame)) {
@@ -718,6 +835,9 @@ extern "C" HRESULT CALLBACK addsymbol(CIDebugClient *client, PCSTR args)
 
     int token;
     const StringVector tokens = commandTokens<StringVector>(args, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     std::string name;
     std::string iname;
     if (tokens.size() >= 2u && integerFromString(tokens.front(), &frame)) {
@@ -747,6 +867,9 @@ extern "C" HRESULT CALLBACK addwatch(CIDebugClient *client, PCSTR argsIn)
     bool success = false;
     do {
         StringList tokens = commandTokens<StringList>(argsIn, &token);
+        if (token == 0) // partial message
+            return S_OK;
+
         if (tokens.size() != 2) {
             errorMessage = singleLineUsage(commandDescriptions[CmdAddWatch]);
             break;
@@ -776,55 +899,77 @@ extern "C" HRESULT CALLBACK assign(CIDebugClient *client, PCSTR argsIn)
     ExtensionCommandContext exc(client);
 
     std::string errorMessage;
-    bool success = false;
-    AssignEncoding enc = AssignPlainValue;
+    bool encoded = false;
+    bool evaluateExpression = false;
     int token = 0;
-    do {
-        StringList tokens = commandTokens<StringList>(argsIn, &token);
-        if (tokens.empty()) {
-            errorMessage = singleLineUsage(commandDescriptions[CmdAssign]);
-            break;
-        }
-
+    StringList tokens = commandTokens<StringList>(argsIn, &token);
+    while (!tokens.empty()) {
         if (tokens.front() == "-h") {
-            enc = AssignHexEncoded;
+            encoded = true;
             tokens.pop_front();
-        } else if (tokens.front() == "-u") {
-            enc = AssignHexEncodedUtf16;
+            continue;
+        }
+
+        if (tokens.front() == "-e") {
+            evaluateExpression = true;
             tokens.pop_front();
+            continue;
         }
+        break;
+    };
 
-        if (tokens.empty()) {
-            errorMessage = singleLineUsage(commandDescriptions[CmdAssign]);
-            break;
+    // Parse 'assign locals.x=5'
+    const std::string::size_type equalsPos = tokens.empty() ? std::string::npos
+                                                            : tokens.front().find('=');
+    if (equalsPos == std::string::npos) {
+        errorMessage = singleLineUsage(commandDescriptions[CmdAssign]);
+    } else {
+        std::string iname = tokens.front().substr(0, equalsPos);
+        const std::string value = tokens.front().substr(equalsPos + 1,
+                                                        tokens.front().size() - equalsPos - 1);
+        SymbolGroup *symGroup = nullptr;
+        if (evaluateExpression) {
+            WatchesSymbolGroup *watchesSymGroup
+                = ExtensionContext::instance().watchesSymbolGroup(exc.symbols(), &errorMessage);
+            std::string tempAssignIname = "watch.tmpassign";
+            if (watchesSymGroup) {
+                if (watchesSymGroup->addWatch(exc.symbols(),
+                                              tempAssignIname,
+                                              stringFromHex(iname),
+                                              &errorMessage)) {
+                    iname = tempAssignIname;
+                    symGroup = watchesSymGroup;
+                }
+            }
+        } else {
+            // get the symbolgroup
+            int currentFrame = ExtensionContext::instance().symbolGroupFrame();
+            if (currentFrame < 0) {
+                CIDebugControl *control = ExtensionCommandContext::instance()->control();
+                DEBUG_STACK_FRAME frame;
+                if (FAILED(control->GetStackTrace(0, 0, 0, &frame, 1, NULL)))
+                    errorMessage = "No current frame.";
+                else
+                    currentFrame = frame.FrameNumber;
+            }
+            if (currentFrame >= 0) {
+                symGroup = ExtensionContext::instance().symbolGroup(exc.symbols(),
+                                                                    exc.threadId(),
+                                                                    currentFrame,
+                                                                    &errorMessage);
+            }
         }
+        if (symGroup
+            && symGroup->assign(iname,
+                                encoded ? stringFromHex(value) : value,
+                                SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()),
+                                &errorMessage)) {
+            ExtensionContext::instance().report('R', token, 0, "assign", "Ok");
+            return S_OK;
+        }
+    }
 
-        // Parse 'assign locals.x=5'
-        const std::string::size_type equalsPos = tokens.front().find('=');
-        if (equalsPos == std::string::npos) {
-            errorMessage = singleLineUsage(commandDescriptions[CmdAssign]);
-            break;
-        }
-        const std::string iname = tokens.front().substr(0, equalsPos);
-        const std::string value = tokens.front().substr(equalsPos + 1, tokens.front().size() - equalsPos - 1);
-        // get the symbolgroup
-        const int currentFrame = ExtensionContext::instance().symbolGroupFrame();
-        if (currentFrame < 0) {
-            errorMessage = "No current frame.";
-            break;
-        }
-        SymbolGroup *symGroup = ExtensionContext::instance().symbolGroup(exc.symbols(), exc.threadId(), currentFrame, &errorMessage);
-        if (!symGroup)
-            break;
-        success = symGroup->assign(iname, enc, value,
-                                   SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()),
-                                   &errorMessage);
-    } while (false);
-
-    if (success)
-        ExtensionContext::instance().report('R', token, 0, "assign", "Ok");
-    else
-        ExtensionContext::instance().report('N', token, 0, "assign", errorMessage.c_str());
+    ExtensionContext::instance().report('N', token, 0, "assign", errorMessage.c_str());
     return S_OK;
 }
 
@@ -838,6 +983,8 @@ extern "C" HRESULT CALLBACK threads(CIDebugClient *client, PCSTR  argsIn)
 
     int token;
     commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial message
+        return S_OK;
 
     const std::string gdbmi = gdbmiThreadList(exc.systemObjects(),
                                               exc.symbols(),
@@ -861,6 +1008,9 @@ extern "C" HRESULT CALLBACK registers(CIDebugClient *Client, PCSTR argsIn)
 
     int token;
     const StringList tokens = commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     const bool humanReadable = !tokens.empty() && tokens.front() == "-h";
     const std::string regs = gdbmiRegisters(exc.registers(), exc.control(), humanReadable, IncludePseudoRegisters, &errorMessage);
     if (regs.empty())
@@ -880,6 +1030,9 @@ extern "C" HRESULT CALLBACK modules(CIDebugClient *Client, PCSTR argsIn)
 
     int token;
     const StringList tokens = commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     const bool humanReadable = !tokens.empty() && tokens.front() == "-h";
     const std::string modules = gdbmiModules(exc.symbols(), humanReadable, &errorMessage);
     if (modules.empty())
@@ -904,6 +1057,9 @@ extern "C" HRESULT CALLBACK setparameter(CIDebugClient *, PCSTR args)
 {
     int token;
     StringVector tokens = commandTokens<StringVector>(args, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     const size_t count = tokens.size();
     size_t success = 0;
     for (size_t i = 0; i < count; ++i) {
@@ -917,6 +1073,12 @@ extern "C" HRESULT CALLBACK setparameter(CIDebugClient *, PCSTR args)
             } else if (!token.compare(0, equalsPos, "maxArraySize")) {
                 if (integerFromString(value, &ExtensionContext::instance().parameters().maxArraySize))
                     ++success;
+            } else if (!token.compare(0, equalsPos, "firstChance")) {
+                if (integerFromString(value, &ExtensionContext::instance().parameters().firstChanceException))
+                    ++success;
+            } else if (!token.compare(0, equalsPos, "secondChance")) {
+                if (integerFromString(value, &ExtensionContext::instance().parameters().secondChanceException))
+                    ++success;
             } else if (!token.compare(0, equalsPos, "maxStackDepth")) {
                 if (integerFromString(value, &ExtensionContext::instance().parameters().maxStackDepth))
                     ++success;
@@ -940,7 +1102,7 @@ extern "C" HRESULT CALLBACK setparameter(CIDebugClient *, PCSTR args)
 extern "C" HRESULT CALLBACK help(CIDebugClient *, PCSTR)
 {
     std::ostringstream str;
-    str << "### Qt Creator CDB extension built " << __DATE__ << "\n\n";
+    str << "### Qt Creator CDB extension" << "\n\n";
 
     const size_t commandCount = sizeof(commandDescriptions)/sizeof(CommandDescription);
     std::copy(commandDescriptions, commandDescriptions + commandCount,
@@ -950,7 +1112,7 @@ extern "C" HRESULT CALLBACK help(CIDebugClient *, PCSTR)
 }
 
 // Extension command 'memory':
-// Display memory as base64
+// Display memory as hex
 
 extern "C" HRESULT CALLBACK memory(CIDebugClient *Client, PCSTR argsIn)
 {
@@ -963,10 +1125,13 @@ extern "C" HRESULT CALLBACK memory(CIDebugClient *Client, PCSTR argsIn)
     ULONG length = 0;
 
     const StringVector tokens = commandTokens<StringVector>(argsIn, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     if (tokens.size()  == 2
             && integerFromString(tokens.front(), &address)
             && integerFromString(tokens.at(1), &length)) {
-        memory = memoryToBase64(exc.dataSpaces(), address, length, &errorMessage);
+        memory = memoryToHex(exc.dataSpaces(), address, length, &errorMessage);
     } else {
         errorMessage = singleLineUsage(commandDescriptions[CmdMemory]);
     }
@@ -990,6 +1155,9 @@ extern "C" HRESULT CALLBACK expression(CIDebugClient *Client, PCSTR argsIn)
 
     do {
         const StringVector tokens = commandTokens<StringVector>(argsIn, &token);
+        if (token == 0) // partial message
+            return S_OK;
+
         if (tokens.size()  != 1) {
 
             errorMessage = singleLineUsage(commandDescriptions[CmdExpression]);
@@ -1019,6 +1187,9 @@ extern "C" HRESULT CALLBACK stack(CIDebugClient *Client, PCSTR argsIn)
     unsigned maxFrames = ExtensionContext::instance().parameters().maxStackDepth;
 
     StringList tokens = commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     if (!tokens.empty() && tokens.front() == "-h") {
          humanReadable = true;
          tokens.pop_front();
@@ -1050,37 +1221,111 @@ extern "C" HRESULT CALLBACK qmlstack(CIDebugClient *client, PCSTR argsIn)
 
     int token = 0;
     bool humanReadable = false;
-    ULONG64 jsExecutionContext = 0;
+    ULONG64 jsExecutionEngine = 0;
     std::string stackDump;
 
     do {
         StringList tokens = commandTokens<StringList>(argsIn, &token);
+        if (token == 0) // partial message
+            return S_OK;
+
         if (!tokens.empty() && tokens.front() == "-h") {
              humanReadable = true;
              tokens.pop_front();
         }
         if (!tokens.empty()) {
-            if (!integerFromString(tokens.front(), &jsExecutionContext)) {
+            if (!integerFromString(tokens.front(), &jsExecutionEngine)) {
                 errorMessage = "Invalid address " + tokens.front();
                 break;
             }
             tokens.pop_front();
         }
         ExtensionCommandContext exc(client);
-        if (!jsExecutionContext) { // Try to find execution context unless it was given.
-            jsExecutionContext = ExtensionContext::instance().jsExecutionContext(exc, &errorMessage);
-            if (!jsExecutionContext)
+        if (!jsExecutionEngine) { // Try to find execution engine unless it was given.
+            jsExecutionEngine = ExtensionContext::instance().jsExecutionEngine(exc, &errorMessage);
+            if (!jsExecutionEngine)
                 break;
         }
         // call function to get stack trace. Call with exceptions handled right from
         // the start assuming this is invoked for crashed applications.
-        std::ostringstream callStr;
-        const QtInfo &qtInfo = QtInfo::get(SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()));
-        callStr << qtInfo.prependQtModule("qt_v4StackTrace(", QtInfo::Qml) << std::showbase << std::hex
-                << jsExecutionContext << std::dec << std::noshowbase << ')';
+        // multiple function calls are needed, depending on the used Qt version
+        // We always start from the latest Qt version
+        std::ostringstream stringBuilder;
         std::wstring wOutput;
-        if (!ExtensionContext::instance().call(callStr.str(), ExtensionContext::CallWithExceptionsHandled, &wOutput, &errorMessage))
+        const QtInfo &qtInfo = QtInfo::get(SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()));
+        do {
+            stringBuilder << qtInfo.prependQtModule("qt_v4StackTraceForEngine(", QtInfo::Qml) << std::showbase << std::hex
+                    << jsExecutionEngine << std::dec << std::noshowbase << ')';
+            if (ExtensionContext::instance().call(stringBuilder.str(), ExtensionContext::CallWithExceptionsHandled, &wOutput, &errorMessage))
+                break;
+
+            // < Qt 5.15
+            // We need to retrieve the current Context first
+            std::string currentContextStr;
+
+            // First try calling the currentContext() function
+            std::wstring callResult;
+
+            stringBuilder.str("");
+            stringBuilder << qtInfo.prependQtModule("QV4::ExecutionEngine::currentContext(", QtInfo::Qml) << std::showbase << std::hex
+                          << jsExecutionEngine << std::dec << std::noshowbase << ")";
+            if (ExtensionContext::instance().call(stringBuilder.str(), ExtensionContext::CallWithExceptionsHandled, &callResult, &errorMessage)) {
+                const std::string::size_type sPos = callResult.find(L"struct QV4::ExecutionContext * ") + 31 /*size of pattern*/;
+                const std::string::size_type sEndPos = callResult.find(L'+');
+
+                if (sPos == std::string::npos || sEndPos == std::string::npos || sEndPos < sPos) {
+                    errorMessage = "Couldn't parse address from debugger output";
+                    break;
+                }
+                currentContextStr = wStringToString(callResult.substr(sPos, sEndPos - sPos));
+            } else {
+                // < Qt 5.11 ????
+                // currentContext is a member, not a function
+
+                stringBuilder.str("");
+                stringBuilder << "((QV4::ExecutionEngine*)" << std::showbase << std::hex
+                              << jsExecutionEngine << std::dec << std::noshowbase << ")->currentContext";
+
+                CIDebugControl *control = ExtensionCommandContext::instance()->control();
+                ULONG oldExpressionSyntax;
+                control->GetExpressionSyntax(&oldExpressionSyntax);
+                control->SetExpressionSyntax(DEBUG_EXPR_CPLUSPLUS);
+
+                IDebugSymbolGroup2 *symbolGroup = nullptr;
+                CIDebugSymbols *symbols = ExtensionCommandContext::instance()->symbols();
+                if (FAILED(symbols->GetScopeSymbolGroup2(DEBUG_SCOPE_GROUP_ALL, NULL,
+                                                         &symbolGroup)))
+                    break;
+
+                ULONG index = DEBUG_ANY_ID;
+                HRESULT hr = symbolGroup->AddSymbol(stringBuilder.str().c_str(), &index);
+                control->SetExpressionSyntax(oldExpressionSyntax);
+                if (SUCCEEDED(hr)) {
+                    ULONG64 address = 0;
+                    HRESULT hr = symbolGroup->GetSymbolOffset(index, &address);
+                    if (SUCCEEDED(hr)) {
+                        ExtensionCommandContext::instance()->dataSpaces()->ReadPointersVirtual(1, address, &address);
+                        stringBuilder.str("");
+                        stringBuilder << std::showbase << std::hex << address;
+                        currentContextStr = stringBuilder.str();
+                    }
+                }
+            }
+
+            if (currentContextStr.empty()) {
+                errorMessage = "Failed to retrieve currenContext from QML engine";
+                break;
+            }
+
+            stringBuilder.str("");
+            stringBuilder << qtInfo.prependQtModule("qt_v4StackTrace(", QtInfo::Qml) << currentContextStr << ')';
+            if (ExtensionContext::instance().call(stringBuilder.str(), ExtensionContext::CallWithExceptionsHandled, &wOutput, &errorMessage))
+                break;
+        } while (false);
+
+        if (wOutput.empty())
             break;
+
         // extract GDBMI info from call
         const std::string::size_type sPos = wOutput.find(L"stack=[");
         const std::string::size_type sEndPos = wOutput.rfind(L']');
@@ -1119,6 +1364,9 @@ extern "C" HRESULT CALLBACK widgetat(CIDebugClient *client, PCSTR argsIn)
         int y = -1;
 
         const StringVector tokens = commandTokens<StringVector>(argsIn, &token);
+        if (token == 0) // partial message
+            return S_OK;
+
         if (tokens.size() != 2) {
             errorMessage = singleLineUsage(commandDescriptions[CmdWidgetAt]);
             break;
@@ -1146,6 +1394,9 @@ extern "C" HRESULT CALLBACK breakpoints(CIDebugClient *client, PCSTR argsIn)
     bool humanReadable = false;
     unsigned verbose = 0;
     StringList tokens = commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     while (!tokens.empty() && tokens.front().size() == 2 && tokens.front().at(0) == '-') {
         switch (tokens.front().at(1)) {
         case 'h':
@@ -1175,6 +1426,9 @@ extern "C" HRESULT CALLBACK test(CIDebugClient *client, PCSTR argsIn)
     Mode mode = Invalid;
     int token = 0;
     StringList tokens = commandTokens<StringList>(argsIn, &token);
+    if (token == 0) // partial message
+        return S_OK;
+
     // Parse away options
     while (!tokens.empty() && tokens.front().size() == 2 && tokens.front().at(0) == '-') {
         const char option = tokens.front().at(1);
