@@ -27,17 +27,23 @@
 #include "bookmarkmanager.h"
 #include "bookmarks_global.h"
 
+#include <utils/utilsicons.h>
+
+#include <QApplication>
 #include <QFileInfo>
 #include <QTextBlock>
 
 using namespace Bookmarks::Internal;
+using namespace Utils;
 
 Bookmark::Bookmark(int lineNumber, BookmarkManager *manager) :
-    TextMark(QString(), lineNumber, Constants::BOOKMARKS_TEXT_MARK_CATEGORY),
+    TextMark(FilePath(), lineNumber, Constants::BOOKMARKS_TEXT_MARK_CATEGORY),
     m_manager(manager)
 {
+    setColor(Utils::Theme::Bookmarks_TextMarkColor);
+    setIcon(Utils::Icons::BOOKMARK_TEXTEDITOR.icon());
+    setDefaultToolTip(QApplication::translate("BookmarkManager", "Bookmark"));
     setPriority(TextEditor::TextMark::NormalPriority);
-    setIcon(m_manager->bookmarkIcon());
 }
 
 void Bookmark::removedFromEditor()
@@ -45,6 +51,16 @@ void Bookmark::removedFromEditor()
     m_manager->deleteBookmark(this);
 }
 
+bool Bookmark::isDraggable() const
+{
+    return true;
+}
+
+void Bookmark::dragToLine(int lineNumber)
+{
+    move(lineNumber);
+}
+
 void Bookmark::updateLineNumber(int line)
 {
     if (line != lineNumber()) {
@@ -58,27 +74,31 @@ void Bookmark::move(int line)
     if (line != lineNumber()) {
         TextMark::move(line);
         m_manager->updateBookmark(this);
+        updateMarker();
     }
 }
 
 void Bookmark::updateBlock(const QTextBlock &block)
 {
-    if (m_lineText != block.text()) {
-        m_lineText = block.text();
+    const QString &lineText = block.text().trimmed();
+    if (m_lineText != lineText) {
+        m_lineText = lineText;
         m_manager->updateBookmark(this);
     }
 }
 
-void Bookmark::updateFileName(const QString &fileName)
+void Bookmark::updateFileName(const FilePath &fileName)
 {
-    const QString &oldFileName = this->fileName();
+    const FilePath &oldFileName = this->fileName();
     TextMark::updateFileName(fileName);
-    m_manager->updateBookmarkFileName(this, oldFileName);
+    m_manager->updateBookmarkFileName(this, oldFileName.toString());
 }
 
 void Bookmark::setNote(const QString &note)
 {
     setToolTip(note);
+    setLineAnnotation(note);
+    updateMarker();
 }
 
 void Bookmark::updateNote(const QString &note)
