@@ -38,7 +38,7 @@
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qlist.h>
-#include <qregexp.h>
+#include <qregularexpression.h>
 #include <qset.h>
 #include <qstack.h>
 #include <qstring.h>
@@ -330,7 +330,7 @@ ProStringList QMakeEvaluator::split_value_list(const QStringRef &vals, int sourc
 }
 
 static void replaceInList(ProStringList *varlist,
-        const QRegExp &regexp, const QString &replace, bool global, QString &tmp)
+        const QRegularExpression &regexp, const QString &replace, bool global, QString &tmp)
 {
     for (ProStringList::Iterator varit = varlist->begin(); varit != varlist->end(); ) {
         QString val = varit->toQString(tmp);
@@ -596,14 +596,16 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::visitProBlock(
         case TokBypassNesting:
             blockLen = getBlockLen(tokPtr);
             if ((m_cumulative || okey != or_op) && blockLen) {
-                ProValueMapStack savedValuemapStack = m_valuemapStack;
+                ProValueMapStack savedValuemapStack = std::move(m_valuemapStack);
                 m_valuemapStack.clear();
-                m_valuemapStack.append(savedValuemapStack.takeFirst());
+                m_valuemapStack.splice(m_valuemapStack.end(),
+                                       savedValuemapStack, savedValuemapStack.begin());
                 traceMsg("visiting nesting-bypassing block");
                 ret = visitProBlock(tokPtr);
                 traceMsg("visited nesting-bypassing block");
-                savedValuemapStack.prepend(m_valuemapStack.first());
-                m_valuemapStack = savedValuemapStack;
+                savedValuemapStack.splice(savedValuemapStack.begin(),
+                                          m_valuemapStack, m_valuemapStack.begin());
+                m_valuemapStack = std::move(savedValuemapStack);
             } else {
                 traceMsg("skipped nesting-bypassing block");
                 ret = ReturnTrue;
@@ -891,9 +893,10 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::visitProVariable(
         QString pattern = func[1].toString();
         QString replace = func[2].toString();
         if (quote)
-            pattern = QRegExp::escape(pattern);
+            pattern = QRegularExpression::escape(pattern);
 
-        QRegExp regexp(pattern, case_sense ? Qt::CaseSensitive : Qt::CaseInsensitive);
+        QRegularExpression regexp(pattern, case_sense ? QRegularExpression::NoPatternOption :
+                                                        QRegularExpression::CaseInsensitiveOption);
 
         // We could make a union of modified and unmodified values,
         // but this will break just as much as it fixes, so leave it as is.
@@ -941,7 +944,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::visitProVariable(
         m_dirSep = first(varName);
     else if (varName == statics.strQMAKESPEC) {
         if (!values(varName).isEmpty()) {
-            QString spec = values(varName).first().toQString();
+            QString spec = values(varName).constFirst().toQString();
             if (IoUtils::isAbsolutePath(spec)) {
                 m_qmakespec = spec;
                 m_qmakespecName = IoUtils::fileName(m_qmakespec).toString();
@@ -1433,7 +1436,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::visitProFile(
 
         for (ProValueMap::ConstIterator it = m_extraVars.constBegin();
              it != m_extraVars.constEnd(); ++it)
-            m_valuemapStack.first().insert(it.key(), it.value());
+            m_valuemapStack.front().insert(it.key(), it.value());
 
         // In case default_pre needs to make decisions based on the current
         // build pass configuration.
@@ -1700,9 +1703,9 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateFunction(
 {
     VisitReturn vr;
 
-    if (m_valuemapStack.count() >= 100) {
+    if (m_valuemapStack.size() >= 100) {
         evalError(fL1S("Ran into infinite recursion (depth > 100)."));
-        vr = ReturnFalse;
+        vr = ReturnError;
     } else {
         m_valuemapStack.push(ProValueMap());
         m_locationStack.push(m_current);
@@ -1850,7 +1853,7 @@ static bool isFunctParam(const ProKey &variableName)
 
 ProValueMap *QMakeEvaluator::findValues(const ProKey &variableName, ProValueMap::Iterator *rit)
 {
-    ProValueMapStack::Iterator vmi = m_valuemapStack.end();
+    ProValueMapStack::iterator vmi = m_valuemapStack.end();
     for (bool first = true; ; first = false) {
         --vmi;
         ProValueMap::Iterator it = (*vmi).find(variableName);
@@ -1870,14 +1873,14 @@ ProValueMap *QMakeEvaluator::findValues(const ProKey &variableName, ProValueMap:
 
 ProStringList &QMakeEvaluator::valuesRef(const ProKey &variableName)
 {
-    ProValueMap::Iterator it = m_valuemapStack.top().find(variableName);
+    ProValueMap::iterator it = m_valuemapStack.top().find(variableName);
     if (it != m_valuemapStack.top().end()) {
         if (it->constBegin() == statics.fakeValue.constBegin())
             it->clear();
         return *it;
     }
     if (!isFunctParam(variableName)) {
-        ProValueMapStack::Iterator vmi = m_valuemapStack.end();
+        ProValueMapStack::iterator vmi = m_valuemapStack.end();
         if (--vmi != m_valuemapStack.begin()) {
             do {
                 --vmi;
@@ -1896,7 +1899,7 @@ ProStringList &QMakeEvaluator::valuesRef(const ProKey &variableName)
 
 ProStringList QMakeEvaluator::values(const ProKey &variableName) const
 {
-    ProValueMapStack::ConstIterator vmi = m_valuemapStack.constEnd();
+    ProValueMapStack::const_iterator vmi = m_valuemapStack.cend();
     for (bool first = true; ; first = false) {
         --vmi;
         ProValueMap::ConstIterator it = (*vmi).constFind(variableName);
@@ -1905,7 +1908,7 @@ ProStringList QMakeEvaluator::values(const ProKey &variableName) const
                 break;
             return *it;
         }
-        if (vmi == m_valuemapStack.constBegin())
+        if (vmi == m_valuemapStack.cbegin())
             break;
         if (first && isFunctParam(variableName))
             break;
@@ -1933,7 +1936,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateFile(
         m_current = m_locationStack.pop();
         pro->deref();
         if (ok == ReturnTrue && !(flags & LoadHidden)) {
-            ProStringList &iif = m_valuemapStack.first()[ProKey("QMAKE_INTERNAL_INCLUDED_FILES")];
+            ProStringList &iif = m_valuemapStack.front()[ProKey("QMAKE_INTERNAL_INCLUDED_FILES")];
             ProString ifn(fileName);
             if (!iif.contains(ifn))
                 iif << ifn;
@@ -1954,7 +1957,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateFileChecked(
         for (const ProFile *pf : ref->m_profileStack)
             if (pf->fileName() == fileName) {
                 evalError(fL1S("Circular inclusion of %1.").arg(fileName));
-                return ReturnFalse;
+                return ReturnError;
             }
     } while ((ref = ref->m_caller));
     return evaluateFile(fileName, type, flags);
@@ -2062,7 +2065,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateFileInto(
         return ret;
     *values = visitor.m_valuemapStack.top();
     ProKey qiif("QMAKE_INTERNAL_INCLUDED_FILES");
-    ProStringList &iif = m_valuemapStack.first()[qiif];
+    ProStringList &iif = m_valuemapStack.front()[qiif];
     const auto ifns = values->value(qiif);
     for (const ProString &ifn : ifns)
         if (!iif.contains(ifn))
